/*  QT4D Player - An ImageJ Plug-in that Plays 4D Quicktime Movies
	
	By Jeff and Jon Hardin
	
	Opens a quicktime movie and allows 4D navigation.
	Based on QTGrabMv plug-in by Bob Dougherty.
	The starting point was QT_Capture by Wayne Rasband.
	Thanks to Bob Dougherty for help with tips on parameter passing and KeyListeners.
	Brightness and contrast support is now cross-platform compatible with Mac OS X (10.4).
	
	June 19th, 2004
    Additional draft revision for resizing by dragging - Sept 23, 2004 by Jeff Hardin
    Additional changes Jan. 30, 2005 by Jeff Hardin	
	Version 0.5.1, 6/21/2005
	
	Version 0..5.2. 10/11/2007 - fixed focus issues under OS X

*/


import ij.*;
import ij.plugin.*;
import ij.gui.*;
import ij.io.*;
import ij.process.*;
import java.awt.*;
import java.util.Date;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;

import quicktime.*;
import quicktime.io.*;
import quicktime.qd.*;
import quicktime.std.clocks.*;
import quicktime.std.image.*;
import quicktime.std.movies.*;
import quicktime.std.movies.media.*;
import quicktime.std.movies.media.VideoMediaHandler;
import quicktime.app.time.*;
import quicktime.std.image.GraphicsImporter;
import quicktime.std.image.GraphicsMode;
import quicktime.app.display.*;
import quicktime.util.*;
import quicktime.std.StdQTConstants;
import quicktime.std.StdQTConstants4;
import quicktime.std.StdQTConstants5;
import quicktime.std.StdQTConstants6;
import quicktime.app.view.*;

public class QT_4D_Player implements PlugIn
{
	int count;

	public void run(String arg)
	{
		try
		{
			// Start up Quicktime and create a QT4DMovie object, showing it assuming the user didn't cancel:
			
			QTSession.open();
			QT4DMovie sgm = new QT4DMovie("Movie");
			if (!sgm.canceled)
			{
				sgm.show();
				sgm.toFront();
			}
		}
		catch (QTException e)
		{
			// Quicktime threw an exception!
			e.printStackTrace();
			QTSession.close ();
		}
		catch (NoClassDefFoundError e)
		{
			// Quicktime is not installed!
			e.printStackTrace();
		}
	}
}


class QT4DMovie extends Frame implements WindowListener, ActionListener, ItemListener, MouseListener, MouseMotionListener, MouseWheelListener, KeyListener, AdjustmentListener, FocusListener, ComponentListener, StdQTConstants, Errors, QDConstants, MovieDrawingComplete
{

	private Button HomeButton, EndButton, PlayButton, MagButton;
	private Cursor ArrowCursor, UpCursor, DownCursor, LeftCursor, RightCursor, HandCursor, MoveCursor, magCursor;
    private Scrollbar MyScrollBar;
    //private Choice MagChoice;
	private Button StepFwdButton, StepBwdButton, MyUpButton, MyDownButton;
    public int currentTime, currentPlane, totalPlanes, totalTimePoints, framesPerFocalPlane,currentMag;
    public double magArray[];
    public String magText[];
    public String origTitle;
    public double origx,origy,origwidth,origheight;
    public boolean playingMovie;
	QTImageProducer qtip;
	QTFile qtf = null;
	int i, numFrames, totalFrames, nextTime, nowTime, count;
    int totalResizeEvents;
	Image img;
	boolean grayscale;
    TimeInfo timeInfo;
	public QTComponent qtComponent;
    public Graphics g;
	private MoviePlayer myPlayer;
	public Label planeLabel, timeLabel, zoomLabel;
	//added for resizing 8-16-04
	//Panel OuterPanel, moviePanel;
	Panel MyPanel;
	boolean needsRedrawing;
    boolean startedDragging = false;
    boolean mousePress = false;
	Track imageTrack;
	Movie m;
    public String movieName,infoString,timeString,spacingString;
	int width, height, currentOffsetX, currentOffsetY, dragOffsetX, dragOffsetY, oldOffsetX, oldOffsetY;
    int startWidth,startHeight,origMag;
	public int[] frameTime;
	boolean stackNeedsReplacing = true;
	ImagePlus imp;
	ImageStack stack;
	boolean canceled = false;
	QT4DMonitor theMoviesExtremeCallback;
    Dimension origDim,startDim;
    public QDRect origMovieRect;
	public PopupMenu pm;
    private Color myColor;
    boolean cancelbutton = false;
    boolean changingZoom = false;
    boolean mouseIsDown = false;
    private javax.swing.Timer myTimer, resizeTimer;
    int mousePressX = 0;
    int mousePressY = 0;
    Object mousePressObject;
    public int tempID = 1;
    public boolean spaceBarPressed = false;
    private MenuBar mb = new MenuBar();
    private Menu fileMenu,editMenu;
    public int copyCounter = 0;
    public int totalToCopy = 1;
    public Component myComponent;
    int INT_MAX = 1000000;
	//next two lines added to remember locations and sizes when movie initially opened
	public Point origMovieLocation,origScrollLocation,origPanelLocation;
	public Dimension origFrameSize, origComponentSize, origScrollSize;

	QT4DMovie(String title)
	{
		// The constructor sets up the user interface for the window and asks the user for
		// a movie to open and display in four dimensions.
		
		super(title);
        Menu mm = new Menu("File");
        mm.add(new MenuItem("Close", new MenuShortcut(KeyEvent.VK_W)));
        mm.add(new MenuItem("--"));
        mm.add(new MenuItem("Get Info...", new MenuShortcut(KeyEvent.VK_I)));
		mm.add(new MenuItem("--"));
		mm.add(new MenuItem("Show/Hide Control Panel"));
        mm.add(new MenuItem("--"));
        mm.add(new MenuItem("About This Plugin..."));;
        mm.addActionListener(this);
        fileMenu = mm;
        mb.add(mm);
        
        Menu em = new Menu("Edit");
        em.add("Copy Focal Plane to Stack");
        em.add("Copy Time Point to Stack");
        em.add(new MenuItem("Copy Frame to Image Window"));
		em.add(new MenuItem("Apply Filter..."));
        em.add(new MenuItem("Revert to Original"));

        em.addActionListener(this);
        editMenu = em;
        mb.add(em);

        setMenuBar(mb);
		setResizable(true);
		// This adds a resize detector:
        addComponentListener(this);
		//Added 10/11/07:
		addFocusListener(this);
		//
        currentOffsetX = 0;
        currentOffsetY = 0;
        dragOffsetX = 0;
        dragOffsetY = 0;
        totalResizeEvents = 0;
 		timeInfo = new TimeInfo(0, 0);
 		
 		magArray = new double[22];
 		magArray[0] = 0.03125;
 		magArray[1] = 0.0416666667;
 		magArray[2] = 0.0625;
 		magArray[3] = 0.0833333333;
 		magArray[4] = 0.125;
 		magArray[5] = 0.1666666667;
 		magArray[6] = 0.25;
 		magArray[7] = 0.3333333333;
 		magArray[8] = 0.5;
 		magArray[9] = 0.75;
        magArray[10] = 1.0;
		magArray[11] = 1.25;
		magArray[12] = 1.5;
        magArray[13] = 2.0;
        magArray[14] = 3.0;
        magArray[15] = 4.0;
        magArray[16] = 6.0;
        magArray[17] = 8.0;
        magArray[18] = 12.0;
        magArray[19] = 16.0;
        magArray[20] = 24.0;
        magArray[21] = 32.0;
        
		/*
        magText = new String[22];
        magText[0] = "3.1%";
        magText[1] = "4.2%";
        magText[2] = "6.3%";
        magText[3] = "8.3%";
        magText[4] = "12.5%";
        magText[5] = "16.7%";
        magText[6] = "25%";
        magText[7] = "33.3%";
        magText[8] = "50%";
        magText[9] = "75%";
        magText[10] = "100%";
        magText[11] = "125%";
        magText[12] = "150%";		
        magText[13] = "200%";
        magText[14] = "300%";
        magText[15] = "400%";
        magText[16] = "600%";
        magText[17] = "800%";
        magText[18] = "1200%";
        magText[19] = "1600%";
        magText[20] = "2400%";
        magText[21] = "3200%";
		**/

		try
		{
			// Ask the user to identify a movie file to open:
			qtf = QTFile.standardGetFilePreview(QTFile.kStandardQTFileTypes);
            cancelbutton = true;
			if (qtf == null) shutDown();

			String myName = qtf.getName();
			File myPath = qtf.getParentFile();
			totalPlanes = -1;
			
			// The next call sets totalPlanes to a positive int if there's an info file:
			getInfoFile(myPath,myName);
			OpenMovieFile openMovieFile = OpenMovieFile.asRead(qtf);
			if(openMovieFile == null)
			{
				shutDown();
			}
            
            // Open the file, and bail if it fails to load.
			m = Movie.fromFile (openMovieFile);
			if (m == null) shutDown();
            
            // Use the high quality setting for DV - courtesy Bob Dougherty:
			m.setPlayHints(StdQTConstants.hintsHighQuality, StdQTConstants.hintsHighQuality);

			int numTracks = m.getTrackCount();
			if (numTracks < 1) shutDown();
			int trackMostLikely = 0;
			int trackNum = 0;
			
			// Loop through the movie's tracks to find the video track:
			while((++trackNum <= numTracks) && (trackMostLikely == 0))
			{
				imageTrack = m.getTrack(trackNum);
				QDDimension d = imageTrack.getSize();
				if (d.getWidth() > 0)
				{
					 // The first track with width != soundtrack.
					 trackMostLikely = trackNum;
				}
			}
			
			// Extract the video track from the movie:
			imageTrack = m.getTrack(trackMostLikely);
			QDDimension d = imageTrack.getSize();
			width = d.getWidth();
			height = d.getHeight();
            startWidth = width;
            startHeight = height;
            
			// Check to see if movie fits on screen:
        	Dimension theScreen = Toolkit.getDefaultToolkit().getScreenSize();
			double screenWidth = theScreen.width;
        	double screenHeight = theScreen.height;
        	origMag = 10;
         	
         	// If necessary, adjust the movie size to fit:
         	if ((width > (int) (0.9*screenWidth)) || (height > (int) (0.9*screenHeight)))
         	{
				IJ.showMessage("Movie exceeds screen dimensions. Size will be adjusted.");
				int tempWidth = width; int tempHeight = height;
				int tempIndex = 10;
				for (int i = 1; i < 11; i++)
				{
				    tempIndex = tempIndex - 1;
				    tempWidth = (int) (width * magArray[10-i]);
				    tempHeight = (int) (height * magArray[10-i]);
				    if ((tempWidth < (int) (0.9*screenWidth)) & (tempHeight < (int) (0.9*screenHeight)))
				    {
				    	break;
				    }
				}
				origMag = tempIndex;
				//changeMagMenu(origMag);
				//MagChoice.select(origMag);
				QDRect newBox = new QDRect(tempWidth,tempHeight);
				m.setBox(newBox);
			} 
			
			//IJ.log("Set mag choice. tempIndex = " + origMag);
                       
            // Open a QTCanvas that can't be resized. - JDH
			// Let's try allow aspect ratio (integer ratio) resizing!
                        
            qtComponent = QTFactory.makeQTComponent(m);
            myComponent = (Component) qtComponent;
			myComponent.addFocusListener(this);
			/*
            // Instantiate a Java panel with a Quicktime movie:
			moviePanel = new Panel();
            moviePanel.add(myComponent);
			**/			
			myPlayer = new MoviePlayer (m);

			currentTime = 1;
			currentPlane = 1;
			totalTimePoints = 1000000;
			currentMag = 10;
            movieName = myName;

			// Attach the popup menu component to the main window:
			addPopupMenu();

            MyPanel = new Panel();
            String MyString = "Focal plane: " + currentPlane + "     ";
            planeLabel = new Label(MyString, Label.LEFT);
            MyPanel.add(planeLabel);
            MyString = "Time point: " + currentTime + "      ";
            timeLabel = new Label(MyString, Label.LEFT);
            MyPanel.add(timeLabel);
            MyString = "Zoom: " + "???%" + "      ";

			// Add all of the buttons to the interface:

			HomeButton = new Button("<|");
			MyPanel.add(HomeButton);
			HomeButton.addActionListener(this);
			HomeButton.addMouseListener(this);

			StepBwdButton = new Button();
			StepBwdButton.setLabel("<<");
			MyPanel.add(StepBwdButton);
			StepBwdButton.addActionListener(this);
			StepBwdButton.addMouseListener(this);

			PlayButton = new Button();
			PlayButton.setLabel(">");
			MyPanel.add(PlayButton);
			PlayButton.addActionListener(this);
			PlayButton.addMouseListener(this);

			StepFwdButton = new Button();
			MyPanel.add(StepFwdButton);
			StepFwdButton.setLabel(">>");
			StepFwdButton.addActionListener(this);
            StepFwdButton.addMouseListener(this);

			EndButton = new Button("|>");
			MyPanel.add(EndButton);
			EndButton.addActionListener(this);
			EndButton.addMouseListener(this);

            MyUpButton = new Button();
            MyUpButton.setLabel("^");
			MyPanel.add(MyUpButton);
			MyUpButton.addActionListener(this);
			MyUpButton.addMouseListener(this);

            MyDownButton = new Button();
            MyDownButton.setLabel("v");
			MyPanel.add(MyDownButton);
			MyDownButton.addActionListener(this);
			MyDownButton.addMouseListener(this);

			//Panel OuterPanel = new Panel();
			//had to change to global
			//OuterPanel = new Panel();
			//BorderLayout outerLayout = new BorderLayout();
			//OuterPanel.setLayout(outerLayout);
            MyScrollBar = new Scrollbar(Scrollbar.HORIZONTAL,1,1,1,100);
            MyScrollBar.addAdjustmentListener(this);
			MyScrollBar.setFocusable(false);
            MyScrollBar.setUnitIncrement(1);
            MyScrollBar.setBlockIncrement(1);
			//OuterPanel.add(MyScrollBar,"North");
            //OuterPanel.add(MyPanel,"South");
			//add(OuterPanel,"South");
			//FlowLayout finalLayout = new FlowLayout();
			add(myComponent,"Center");
			add(MyScrollBar,"North");
			add(MyPanel,"South");
			pack();
			setLayout(null);
			//add(OuterPanel);
			//add(OuterPanel,"South");
			//add(moviePanel,"Center");
			//had next line in for dynamic resizing; unreliable!
			//add(myComponent,"Center");
			//pack();
			
			//myComponent.setLocation(0,0);
			myComponent.setLocation(0,this.getInsets().top);
			//MyScrollBar.setLocation(0,myComponent.getHeight()+1);
			MyScrollBar.setLocation(0,this.getInsets().top+myComponent.getHeight()+1);
			//MyPanel.setLocation(0,MyScrollBar.getHeight()+1);
			MyPanel.setLocation(0,this.getInsets().top+MyScrollBar.getHeight()+1);
			/*
			Rectangle myBounds = myComponent.getBounds();
			int clipX = myBounds.x;
			int clipY = myBounds.y;
			int clipWidth = myBounds.width;
			int clipHeight = myBounds.height;
			QDRect clip = new QDRect(clipX,clipY,clipWidth,clipHeight);
			myPlayer.setClip(new Region(clip));
			**/

            ArrowCursor = new Cursor ( Cursor.DEFAULT_CURSOR );
			UpCursor = new Cursor ( Cursor.N_RESIZE_CURSOR );
			DownCursor = new Cursor ( Cursor.S_RESIZE_CURSOR );
			RightCursor = new Cursor ( Cursor.E_RESIZE_CURSOR );
			LeftCursor = new Cursor ( Cursor.W_RESIZE_CURSOR );
			HandCursor = new Cursor ( Cursor.HAND_CURSOR );
			MoveCursor = new Cursor ( Cursor.MOVE_CURSOR );
			magCursor = new Cursor ( Cursor.NE_RESIZE_CURSOR);

			// Add control listeners to the window:

			addWindowListener(this);
            addMouseListener(this);
            addMouseMotionListener(this);
			addMouseWheelListener(this);
			addKeyListener(this);
                        
            myComponent.addMouseListener(this);
			myComponent.addKeyListener(this);
            myComponent.addMouseMotionListener(this);
			myComponent.addMouseWheelListener(this);
            
            int maxTime = m.getDuration();
  			timeInfo = new TimeInfo(0, 0);
  			m.setTimeValue(0);
			totalFrames = 0;
			frameTime = new int[INT_MAX];
			
			// Count the total number of frames:					// CHANGE MADE:
			do														// The frame processing loop was combined in with
			{														// the frame counter loop, with a variable-sized array.
				frameTime[totalFrames] = timeInfo.time;	
				totalFrames++;
				timeInfo = imageTrack.getNextInterestingTime(nextTimeMediaSample, timeInfo.time, 1f);
			}
			while (timeInfo.time > -1);
			
            IJ.showStatus("Total frames: " + totalFrames);
            totalTimePoints = totalFrames;
            
            m.setTimeValue(0);
			i = 0;
			nextTime = 0;
            nowTime = 0;
            playingMovie = false;
            origDim = myComponent.getSize();
            Container tempContainer = myComponent.getParent();
                        
		    startDim = tempContainer.getSize();
		    this.setTitle(myName);origTitle = this.getTitle();
            TimeBase theMoviesTimeBase = m.getTimeBase();
            
            // This callback is triggered when the movie gets to the end:
			theMoviesExtremeCallback = new QT4DMonitor(theMoviesTimeBase, StdQTConstants.triggerAtStop, this );
		    theMoviesExtremeCallback.callMeWhen ();
		    
			// Using the Timer class you can get rescheduled properly and get callbacks at the set intervals. It uses the same callback
			// mechanism of internally of the TimeCallback. It's recomended by Apple to use this Timer class to do callbacks,
			// which would take care of the time base time changes and recscheduling of the tickle method.

			quicktime.app.time.Timer QT4Dtimer = new quicktime.app.time.Timer (4, 1, new QT4DTickler(this), m);
			QT4Dtimer.setActive(true);

			// Get the number of focal planes from the user if they couldn't be pulled from the info file:
            if (totalPlanes < 1)
            {
            	double NumPlanes = IJ.getNumber("Couldn't get focal planes from file. Enter number of focal planes: ", 1);
              	if (NumPlanes < 1)
              	{
              		totalPlanes =1;
              	}
              	else
              	{
              		totalPlanes = (int) NumPlanes;
              	}
            }

			// We can't have more focal planes than movie frames!
            if (totalPlanes > totalTimePoints)
            {
            	IJ.showMessage ("Focal planes exceeds total frames. Reset to " + totalTimePoints);
            	totalPlanes = totalTimePoints;
            }

            framesPerFocalPlane = totalFrames/totalPlanes;
            MyScrollBar.setMaximum(framesPerFocalPlane+1);
            setMyMovieLimits();
            needsRedrawing = true;
            
            // Make sure that focus goes back to the canvas:
            myComponent.requestFocus();

			/*
			// Adjust the movie size if it doesn't fit onscreen:
      		String tempTitle = origTitle;
         	if (origMag != 10)
         	{
         		this.setTitle(tempTitle + " (" + magText[origMag] + ")");
         	}
			**/
        	changeZoom();
       		origMovieRect = myPlayer.getDisplayBounds();
			//IJ.log("Orig rect: X=" + origMovieRect.getX() + " Y=" + origMovieRect.getY() + " W=" + origMovieRect.getWidth() + " H=" + origMovieRect.getHeight());
			//Below for resetting to original size later
			origMovieLocation = myComponent.getLocation();
			origScrollLocation = MyScrollBar.getLocation();
			origPanelLocation = MyPanel.getLocation();
			origFrameSize = this.getSize();
			origComponentSize = myComponent.getSize();
			origScrollSize = MyScrollBar.getSize();
			
	 	}
		catch(QTException e)
		{
			if (cancelbutton) shutDown();
			canceled = true;
		}
		catch (NoClassDefFoundError e)
		{
			e.printStackTrace();
			canceled = true;
		}
	}

	public void addPopupMenu()
	// This method creates a pop-up menu and attaches it to the window.
	{
        pm = new PopupMenu();
        addPopupItem("Copy Focal Plane to Stack");
        addPopupItem("Copy Time Point to Stack");
        addPopupItem("Copy Frame to Image Window");
        addPopupItem("Reset Window to Original Size");
		addPopupItem("Apply Filter...");
        addPopupItem("Revert to Original");
		//added 11-13-04
		addPopupItem("--");
		addPopupItem("Show/Hide Control Panel");
		//
        addPopupItem("--");
        addPopupItem("Get Info...");
        addPopupItem("--");
        addPopupItem("About This Plugin...");

        this.add(pm);
    }

    public void addPopupItem(String s)
    // This adds a pop-up item with title "s" to the pop-up menu "pm".
    {
        MenuItem mi=new MenuItem(s);
        mi.addActionListener(this);
        pm.add(mi);
    }
	
	    Track effectsTrack;
	boolean defined_effects = false;
	double brightness = 0.0;
    double contrast = 0.0;
	
	public void FilterBuilder() throws QTException
    {
        // Check to see if there is already an existing effect applied:
        if (defined_effects == true)
        {
        	m.removeTrack(effectsTrack);
        }
        else
        {
        	defined_effects = true;
        }
		
		Pict[] previewPicts = new Pict[1];
		previewPicts[0] = m.getPict(m.getTime());
        
        // Calculate the movie dimensions:
        QDDimension d = imageTrack.getSize();
		int EFFECT_TRACK_WIDTH = d.getWidth();
        int EFFECT_TRACK_HEIGHT = d.getHeight();
        int TIMESCALE = 600;
        
        // IJ.showMessage("Width: " + EFFECT_TRACK_WIDTH);
        // IJ.showMessage("Height: " + EFFECT_TRACK_HEIGHT);
        
        effectsTrack = m.addTrack(EFFECT_TRACK_WIDTH, EFFECT_TRACK_HEIGHT, 0);
        effectsTrack.setLayer(-1);

		VideoMedia effectsMedia = new VideoMedia(effectsTrack, TIMESCALE);
        
        // Set up the input map: 
        AtomContainer inputMap = new AtomContainer();

        int trackRef = effectsTrack.addReference(imageTrack, StdQTConstants.kTrackModifierReference);

        // Add an input reference atom:
        Atom inputAtom = inputMap.insertChild(null, kTrackModifierInput, trackRef, 0);

        // Add the name and the type:
        inputMap.insertChild(inputAtom, kTrackModifierType, 1, 0, EndianOrder.flipNativeToBigEndian32(videoMediaType));
        inputMap.insertChild(inputAtom, kEffectDataSourceType, 1, 0, EndianOrder.flipNativeToBigEndian32(QTUtils.toOSType ("srcA")));
        
		boolean cancelled = true;
		
        try
		{
        	// Get a list of effects:
        	EffectsList effectsList = new EffectsList(1, 1, 0);
        	AtomContainer effect = ParameterDialog.showParameterDialog(effectsList, 0, null, "Pick an effect", previewPicts);
        	
			cancelled = false;
			
			/*
        	// Ask the user to alter the brightness and contrast:
        	GenericDialog gd = new GenericDialog("Frame Rate Options", IJ.getInstance());
            gd.addSlider("Brightness: ", -100, 100, 0);
            gd.addSlider("Contrast: ", -100, 100, 0);
			gd.showDialog();
			
			// When the user is finished...
			if (gd.wasCanceled())
			{
				defined_effects = false;
        		m.removeTrack(effectsTrack);
        		return;
			}
			else
			{		
                // Retrieve the results from the generic dialog box:
                int tempBrightness = ((int)gd.getNextNumber() + (int)(brightness * 100.0));
                if (tempBrightness > 100)
                {
                	tempBrightness = 100;
                }
                else if (tempBrightness < -100)
                {
                	tempBrightness = -100;
                }
                int tempContrast = ((int)gd.getNextNumber() + (int)(contrast * 100.0));
                if (tempContrast > 100)
                {
                	tempContrast = 100;
                }
                else if (tempContrast < -100)
                {
                	tempContrast = -100;
                }
                brightness = (double)((double)tempBrightness/100.0);
                contrast = (double)((double)tempContrast/100.0);
			}
        	
        	// Determine the Operating System that is running:
        	String os = System.getProperty("os.name").toLowerCase();
        	int effectCode = StdQTConstants4.kBrightnessContrastImageFilterType;
        	AtomContainer effect = new AtomContainer();
        	
        	// Declare constants, which vary by OS:
        	if (os.indexOf("windows") > -1)
        	{
	        	
	        	Atom foo = effect.insertChild(null, 2003329396, 1, 1, EffectsConstants.k_EFFECTS_CODE_WIN32);
	        	Atom foo1 = effect.insertChild(null, 1936876403, 1, 1, EffectsConstants.k_BC_CODE_WIN32);
	        	Atom foo2 = effect.insertChild(null, 1651669364, 1, 1, (int)(EffectsConstants.k_MAX_BRIGHTNESS_WIN32*brightness));
	        	Atom foo3 = effect.insertChild(null, 1668183154, 1, 1, (int)(EffectsConstants.k_MAX_CONTRAST_WIN32*contrast));
        	}
        	else if (os.indexOf("mac") > -1)
        	{
        		Atom foo = effect.insertChild(null, 2003329396, 1, 1, EffectsConstants.k_EFFECTS_CODE_MAC_OSX);
        		Atom foo1 = effect.insertChild(null, 1936876403, 1, 1, EffectsConstants.k_BC_CODE_MAC_OSX);
        		Atom foo2 = effect.insertChild(null, 1651669364, 1, 1, (int)(EffectsConstants.k_MAX_BRIGHTNESS_MAC_OSX*brightness));
        		Atom foo3 = effect.insertChild(null, 1668183154, 1, 1, (int)(EffectsConstants.k_MAX_CONTRAST_MAC_OSX*contrast));
        	}
        	else
        	{
        		// We don't have any other constants defined:
        		defined_effects = false;
        		m.removeTrack(effectsTrack);
        		IJ.showMessage("Effects are unsupported on this platform!");
        		return;
        	}
			*/
        	
        	// Find out the effect type by getting the "what" atom, whose data is a FOUR_CHAR_CODE:
	        Atom what = effect.findChildByIndex_Atom(null, kParameterWhatName, 1);
	        int effectType = effect.getAtomData(what).getInt(0);
	        effectType = EndianOrder.flipBigEndianToNative32(effectType);
	        
			/*
	        // Analyze the primary AtomContainer's contents:
	        int size = effect.getAtomDataSize(what);
	        int ID = effect.getAtomID(what);
	        int type = effect.getAtomType(what);
	        AtomData data = effect.getAtomData(what);
	        int decoded = data.getInt(0);
	        IJ.showStatus("Results: " + size + " " + ID + " " + type + " " + decoded);
	        
	        // Cycle through the atom structure to verify it:
	        for (int i = 0; i < 3; ++i)
	        {
		        int nextCode = effect.getNextChildType(null, type);
		        IJ.showStatus("Next Code: " + nextCode);
		        
		        Atom what2 = effect.findChildByIndex_Atom(null, nextCode, 1);
			    size = effect.getAtomDataSize(what2);
			    ID = effect.getAtomID(what2);
			    type = effect.getAtomType(what2);
			    decoded = effect.getAtomData(what2).getInt(0);
			    IJ.showStatus("Next Results: " + size + " " + ID + " " + type + " " + decoded);
	        }
	        */
	        // Make a sample description for the effect description:
	        ImageDescription imgDesc = ImageDescription.forEffect(effectType);
	        imgDesc.setWidth(EFFECT_TRACK_WIDTH);
	        imgDesc.setHeight(EFFECT_TRACK_HEIGHT);
			
			// Give the effect description a reference to the source:
	        effect.insertChild(null, kEffectSourceName, 1, 0, EndianOrder.flipNativeToBigEndian32(QTUtils.toOSType ("srcA")));
	
			// Add the effect to the video media:
	        effectsMedia.beginEdits();
			effectsMedia.addSample(effect, 0, effect.getSize(), imageTrack.getDuration(), imgDesc, 1, 0);
			effectsMedia.setInputMap(inputMap);
			effectsMedia.endEdits();
	
	        // Now insert this media into track
	        effectsTrack.insertMedia(0, 0, imageTrack.getDuration(), 1);
        }
        catch(QTException e)
        {
        	// This will trap an incompatible movie:
        	defined_effects = false;
        	m.removeTrack(effectsTrack);
        	
			if (cancelled == false)
			{
				IJ.beep(); IJ.showMessage("This movie may have been saved with an earlier version of Quicktime." + "\n" + "Saving as a standalone movie may solve this problem.");
			}
        }
    }
    
    public void RevertFilters() throws QTException
    {
        // Check to see if there is already an existing effect applied:
        if (defined_effects == true)
        {
        	m.removeTrack(effectsTrack);
			defined_effects = false;
        }
        
		/*
        // Calculate the movie dimensions:
        QDDimension d = imageTrack.getSize();
		int EFFECT_TRACK_WIDTH = d.getWidth();
        int EFFECT_TRACK_HEIGHT = d.getHeight();
        int TIMESCALE = 600;
        
        // IJ.showMessage("Width: " + EFFECT_TRACK_WIDTH);
        // IJ.showMessage("Height: " + EFFECT_TRACK_HEIGHT);
        
        effectsTrack = m.addTrack(EFFECT_TRACK_WIDTH, EFFECT_TRACK_HEIGHT, 0);
        effectsTrack.setLayer(-1);

		VideoMedia effectsMedia = new VideoMedia(effectsTrack, TIMESCALE);
        
        // Set up the input map: 
        AtomContainer inputMap = new AtomContainer();

        int trackRef = effectsTrack.addReference(imageTrack, StdQTConstants.kTrackModifierReference);

        // Add an input reference atom:
        Atom inputAtom = inputMap.insertChild(null, kTrackModifierInput, trackRef, 0);

        // Add the name and the type:
        inputMap.insertChild(inputAtom, kTrackModifierType, 1, 0, EndianOrder.flipNativeToBigEndian32(videoMediaType));
        inputMap.insertChild(inputAtom, kEffectDataSourceType, 1, 0, EndianOrder.flipNativeToBigEndian32(QTUtils.toOSType ("srcA")));
        
        try
		{
        	brightness = (double)(0.0);
            contrast = (double)(0.0);
        	
        	// Determine the Operating System that is running:
        	String os = System.getProperty("os.name").toLowerCase();
        	int effectCode = StdQTConstants4.kBrightnessContrastImageFilterType;
        	AtomContainer effect = new AtomContainer();
        	
        	// Declare constants, which vary by OS:
        	if (os.indexOf("windows") > -1)
        	{
	        	
	        	Atom foo = effect.insertChild(null, 2003329396, 1, 1, EffectsConstants.k_EFFECTS_CODE_WIN32);
	        	Atom foo1 = effect.insertChild(null, 1936876403, 1, 1, EffectsConstants.k_BC_CODE_WIN32);
	        	Atom foo2 = effect.insertChild(null, 1651669364, 1, 1, (int)(EffectsConstants.k_MAX_BRIGHTNESS_WIN32*brightness));
	        	Atom foo3 = effect.insertChild(null, 1668183154, 1, 1, (int)(EffectsConstants.k_MAX_CONTRAST_WIN32*contrast));
        	}
        	else if (os.indexOf("mac") > -1)
        	{
        		Atom foo = effect.insertChild(null, 2003329396, 1, 1, EffectsConstants.k_EFFECTS_CODE_MAC_OSX);
        		Atom foo1 = effect.insertChild(null, 1936876403, 1, 1, EffectsConstants.k_BC_CODE_MAC_OSX);
        		Atom foo2 = effect.insertChild(null, 1651669364, 1, 1, (int)(EffectsConstants.k_MAX_BRIGHTNESS_MAC_OSX*brightness));
        		Atom foo3 = effect.insertChild(null, 1668183154, 1, 1, (int)(EffectsConstants.k_MAX_CONTRAST_MAC_OSX*contrast));
        	}
        	else
        	{
        		// We don't have any other constants defined:
        		defined_effects = false;
        		m.removeTrack(effectsTrack);
        		IJ.showStatus("Quicktime EffectsTrack Error...");
        		IJ.showMessage("Effects are unsupported on this platform!");
        		return;
        	}
        	
        	// Find out the effect type by getting the "what" atom, whose data is a FOUR_CHAR_CODE:
	        Atom what = effect.findChildByIndex_Atom(null, kParameterWhatName, 1);
	        int effectType = effect.getAtomData(what).getInt(0);
	        effectType = EndianOrder.flipBigEndianToNative32(effectType);
	        
	        // Analyze the primary AtomContainer's contents:
	        int size = effect.getAtomDataSize(what);
	        int ID = effect.getAtomID(what);
	        int type = effect.getAtomType(what);
	        AtomData data = effect.getAtomData(what);
	        int decoded = data.getInt(0);
	        IJ.showStatus("Results: " + size + " " + ID + " " + type + " " + decoded);
	        
	        // Cycle through the atom structure to verify it:
	        for (int i = 0; i < 3; ++i)
	        {
		        int nextCode = effect.getNextChildType(null, type);
		        IJ.showStatus("Next Code: " + nextCode);
		        
		        Atom what2 = effect.findChildByIndex_Atom(null, nextCode, 1);
			    size = effect.getAtomDataSize(what2);
			    ID = effect.getAtomID(what2);
			    type = effect.getAtomType(what2);
			    decoded = effect.getAtomData(what2).getInt(0);
			    IJ.showStatus("Next Results: " + size + " " + ID + " " + type + " " + decoded);
	        }
	        
	        // Make a sample description for the effect description:
	        ImageDescription imgDesc = ImageDescription.forEffect(effectType);
	        imgDesc.setWidth(EFFECT_TRACK_WIDTH);
	        imgDesc.setHeight(EFFECT_TRACK_HEIGHT);
	
	        // Give the effect description a reference to the source:
	        effect.insertChild(null, kEffectSourceName, 1, 0, EndianOrder.flipNativeToBigEndian32(QTUtils.toOSType ("srcA")));
	
	        // Add the effect to the video media:
	        effectsMedia.beginEdits();
	        effectsMedia.addSample(effect, 0, effect.getSize(), imageTrack.getDuration(), imgDesc, 1, 0);
	        effectsMedia.setInputMap(inputMap);
	        effectsMedia.endEdits();
	
	        // Now insert this media into track
	        effectsTrack.insertMedia(0, 0, imageTrack.getDuration(), 1);
        }
        catch(QTException e)
        {
        	// This will trap a cancellation error from the ParameterDialog:
        	defined_effects = false;
        	m.removeTrack(effectsTrack);
        	IJ.showMessage("Atom Parameter Error!");
        }
		*/
    }

    public void getInfoFile(File thePath, String theName)
    // This checks for the various possible info files that could exist, and tries to open them.
    // If one is found, movie info (such as focal plane count) is retrieved.
    {
		// Test for existence of file first!!
		boolean infoFileExists = false;
		boolean txtFileExists = false;
		boolean infotxtFileExists = false;
		String myName = theName + ".info";
		File myFile = new File(thePath,myName);
		File myFile1 = myFile;
		
		if (myFile.exists())
		{
			infoFileExists = true;
		}
		
		myName = theName + ".txt";
		File myFile2 = new File(thePath,myName);
		if (myFile2.exists())
		{
			txtFileExists = true;
		}
		
		myName = theName + ".info.txt";
		File myFile3 = new File(thePath,myName);
		if (myFile3.exists())
		{
			infotxtFileExists = true;
		}
		
		if (infotxtFileExists) myFile = myFile3;
		if (txtFileExists) myFile = myFile2;
		if (infoFileExists) myFile = myFile1;
		
		if (myFile.exists())
		{
			try
			{
				BufferedReader in = new BufferedReader(new FileReader(myFile));
				String str = "";
				String focalPlaneString = "";
				int lineCount = 0;
				while ((str = in.readLine()) != null)
				{
					lineCount = lineCount + 1;
					if (lineCount == 1)					// CHANGE MADE:
					{									// A single logic gate was made to decrease
						infoString = str;				// the number of statements that resolve in the end.
					}
					else if (lineCount == 2)
					{
						focalPlaneString = str;
					}
					else if (lineCount == 3)
					{
						spacingString = str;
					}
					else if (lineCount == 4)
					{
						timeString = str;
					}
				}
				in.close();
				String trimmedPlanes = focalPlaneString.trim();
				int tempPlanes = Integer.parseInt (trimmedPlanes);
		
				// Set the number of focal planes if it is a valid integer:
				totalPlanes = tempPlanes;
			}
			catch (IOException e)
			{
				printStackTrace(e);
			}
		}
		else
		{
			totalPlanes = -1;
		}
    }

	public double getTimeSinceStart()
	// Figure out how much time has elpased in the movie.
	{
		int movieTime = 0;
		try
		{
			movieTime = m.getTime();
        }
        catch (Exception ee)
        {
        	System.out.println(ee);
        }
        
		double movie2Seconds = (1.d/29.97d)/(frameTime[1] - frameTime[0]);
		return movieTime*movie2Seconds;
	}
	
	public void setTimeSinceStart(double timeSinceStart)
	// Set the position in time points of the movie (not focal planes).
	{
		double movie2Seconds = (1.d/29.97d)/(frameTime[1] - frameTime[0]);
		int movieTime = (int)Math.round(timeSinceStart/movie2Seconds);
		
		try
		{
			m.setTimeValue(movieTime);
        }
        catch (Exception ee)
        {
        	System.out.println(ee);
        }
	}

	public void setMyMovieLimits()
	// Set the limits of the movie in both time points and focal planes, so that we don't walk off.
	{
		try
		{
			int frameNumber1 = (currentPlane - 1) * framesPerFocalPlane + 1;
			int frameNumber2 = (currentPlane - 1) * framesPerFocalPlane + framesPerFocalPlane;
			int myduration = frameTime[frameNumber2 - 1] - frameTime[frameNumber1 - 1] + 1;
			int mystart = frameTime[frameNumber1 - 1];
			TimeInfo mytimeinfo = new TimeInfo(mystart, myduration);
			m.setActiveSegment(mytimeinfo);
			needsRedrawing = true;
        }
        catch (Exception ee)
        {
        	printStackTrace(ee);
        }

	}

	public void setMovieLimitsWideOpen()
	// Set the movie to the beginning and let the limits open to various focal plane settings.
	{
		try
		{	
		    int frameNumber2 = (totalPlanes - 1) * framesPerFocalPlane + framesPerFocalPlane;
			int myduration = frameTime[frameNumber2 - 1] - frameTime[0] + 1;
			int mystart = frameTime[0];
			TimeInfo mytimeinfo = new TimeInfo(mystart, myduration);
			m.setActiveSegment(mytimeinfo);
        }
        catch (Exception ee)
        {
            printStackTrace(ee);
        }
	}

    void stopPlaying()
    // Set the movie's rate to zero.
    {
		try
		{
            float myRate = 0;
            m.setRate(myRate);
        }
        catch (Exception ee)
        {
        	System.out.println(ee);
        }
	}

	public void findNewPosition()
	// Find the current 4D position in focal planes and time points.
	{
		int mytime = 0;
		
		try
		{
			mytime = m.getTime();
			int frameNumber1 = (currentPlane - 1) * framesPerFocalPlane + 1;
			int frameNumber2 = (currentPlane - 1) * framesPerFocalPlane + framesPerFocalPlane;

			int myduration = frameTime[frameNumber2 - 1] - frameTime[frameNumber1 - 1];
			int mystart = frameTime[frameNumber1 - 1];
			double myposition = (100 * (mytime - mystart)/myduration);
			int myslidervalue = (int) (myposition*framesPerFocalPlane/100);
			currentTime = myslidervalue;
			timeLabel.setText("Time point: " + currentTime + "     ");
			int frameNumber = (currentPlane - 1) * framesPerFocalPlane + currentTime;
            m.setTimeValue(frameTime[frameNumber-1]);
            
            if (myslidervalue > 1)
            {
            	MyScrollBar.setValue(currentTime);
            }
		}
		catch (Exception ee)
		{
        	printStackTrace(ee);
        }
	}
	
	public void findRunningPosition()
	{
		// Following repositions the slider at half second intervals while the
		// movie is playing using a Tickle routine.
	
		try
		{
			int mytime = m.getTime();
			int frameNumber1 = (currentPlane - 1) * framesPerFocalPlane + 1;
			int frameNumber2 = (currentPlane - 1) * framesPerFocalPlane + framesPerFocalPlane;
			int myduration = frameTime[frameNumber2 - 1] - frameTime[frameNumber1 - 1];
            int mystart = frameTime[frameNumber1 - 1];
            double myposition = (100 * (mytime - mystart)/myduration);
            int myslidervalue = (int) (myposition*framesPerFocalPlane/100);
            currentTime = myslidervalue;
            
            if (currentTime < 1)
            {
            	currentTime = 1;
            }
            
			timeLabel.setText("Time point: " + currentTime + "     ");
			int frameNumber = (currentPlane - 1) * framesPerFocalPlane + currentTime;
            
            if (myslidervalue > 1)
            {
            	MyScrollBar.setValue(currentTime);
            }
        }
        catch (Exception ee)
        {
        	printStackTrace(ee);
        }
	}

	public void windowOpened(WindowEvent e)
	{
	}

	void shutDown()
	// Dispose of both Quicktime and our main window.
	{
		QTSession.close();
		setVisible(false);
		theMoviesExtremeCallback.cancelAndCleanup();
		dispose();
	}

	void printStackTrace(Exception e)
	// Print out errors to the ImageJ console.
	{
		CharArrayWriter caw = new CharArrayWriter();
		PrintWriter pw = new PrintWriter(caw);
		e.printStackTrace(pw);
		IJ.write(caw.toString());
	}

	void stepFocalPlane(boolean up)									// CHANGE MADE:
	{																// Focal plane manipulation was put in one method.
		try
		{
			needsRedrawing = true;
			float myRate = 0;
            m.setRate(myRate);
			int time = myPlayer.getTime();
			
			if (up)
			{
				currentPlane = currentPlane + 1;
				if (currentPlane > totalPlanes) currentPlane = totalPlanes;
			}
			else
			{
				currentPlane = currentPlane - 1;
				if (currentPlane < 1) currentPlane = 1;
			}
			
			int frameNumber = (currentPlane - 1) * framesPerFocalPlane + currentTime;
			planeLabel.setText("Focal plane: " + currentPlane + "     ");
			
			// You need to set limits wide open to avoid ugliness of frame flashing.
			setMovieLimitsWideOpen();
                        
			m.setTimeValue(frameTime[frameNumber-1]);
			setMyMovieLimits();
        }
        catch (Exception ee)
        {
        	System.out.println(ee);
        }
	}
	
	void stepTimePoint(boolean forward)								// CHANGE MADE:
	{																// Time point manipulation was put in one method.
		try
		{
			needsRedrawing = true;
			float myRate = 0;
            m.setRate(myRate);
			int time = myPlayer.getTime();
			
			if (forward)
			{
				currentTime = currentTime + 1;
				if (currentTime > framesPerFocalPlane) currentTime = framesPerFocalPlane;
			}
			else
			{
				currentTime = currentTime -1;
				if (currentTime < 1) currentTime = 1;
			}
			
			int frameNumber = (currentPlane - 1) * framesPerFocalPlane + currentTime;
			timeLabel.setText("Time point: " + currentTime + "     ");
            m.setTimeValue(frameTime[frameNumber-1]);
            int myTime = MyScrollBar.getValue();            
            if ((currentTime > 0) && (currentTime < MyScrollBar.getMaximum()))
            {
            	MyScrollBar.setValue(currentTime);
            }
        }
        catch (Exception ee)
        {
        	System.out.println(ee);
        }
	}

	void GoFirst()
	// Go to the first time point (not focal plane).
	{
		try
		{
			needsRedrawing = true;
			float myRate = 0;
            m.setRate(myRate);

			int time = myPlayer.getTime();
			currentTime = 1;
			int frameNumber = (currentPlane - 1) * framesPerFocalPlane + currentTime;
			timeLabel.setText("Time point: " + currentTime + "     ");
			m.setTimeValue(frameTime[frameNumber-1]);

			int myTime = MyScrollBar.getValue();
            if (myTime > 1)
            {
            	MyScrollBar.setValue(currentTime);
            }
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

	void GoLast()
	// Go to the last time point (not focal plane).
	{
		try
		{
			needsRedrawing = true;
			float myRate = 0;
            int time = myPlayer.getTime();

			currentTime = framesPerFocalPlane;
			int frameNumber = (currentPlane - 1) * framesPerFocalPlane + currentTime;
			timeLabel.setText("Time point: " + currentTime + "     ");
                        
			m.setTimeValue(frameTime[frameNumber-1]);
			int myTime = MyScrollBar.getValue();
            if (myTime < framesPerFocalPlane)
            {
            	MyScrollBar.setValue(currentTime);
            }
        }
        catch (Exception ee)
        {
        	System.out.println(ee);
        }
	}

	void extremeFocalPlane(boolean top)				// CHANGE MADE:
	// Go to the last focal plane.					// The focal plane navigation became one function.
	{
		try
		{
			needsRedrawing = true;
			float myRate = 0;
            m.setRate(myRate);
			int time = myPlayer.getTime();

			if (top)
			{
				currentPlane = totalPlanes;
			}
			else
			{
				currentPlane = 1;
			}
			
			int frameNumber = (currentPlane - 1) * framesPerFocalPlane + currentTime;
			planeLabel.setText("Focal plane: " + currentPlane + "     ");
			
			// You need to set limits wide open to avoid ugliness of frame flashing.
			setMovieLimitsWideOpen();
            m.setTimeValue(frameTime[frameNumber-1]);
			setMyMovieLimits();
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

	void SlideMovie(int slider)
	// Move the movie to the proper time point, given the positon of the slider.
	{
    	try
    	{
			needsRedrawing = true;
			float myRate = 0;
            m.setRate(myRate);

			int tempTime = slider;
			currentTime = tempTime;
			timeLabel.setText("Time point: " + currentTime + "     ");

			int frameNumber = (currentPlane - 1) * framesPerFocalPlane + currentTime;            
			m.setTimeValue(frameTime[frameNumber-1]);
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

	void alterPosition(int xchange, int ychange)				// CHANGE MADE:
	{															// All of the movement was put in one function.
		try
		{
			needsRedrawing = true;
			currentOffsetY += ychange;
			currentOffsetX += xchange;
			Rectangle canvasRect = myComponent.getBounds();
			QDRect myRect = myPlayer.getDisplayBounds();
			
			if (((ychange == 10) && (currentOffsetY <= 0)) || ((ychange == -10) && ((myRect.getHeight() + currentOffsetY) >= canvasRect.height)) || ((xchange == 10) && (currentOffsetX <= 0)) || ((xchange == -10) && ((myRect.getWidth() + currentOffsetX) >= canvasRect.width)))
			{
				myPlayer.setLocation(currentOffsetX,currentOffsetY);
			}
			else
			{
				currentOffsetY -= ychange;
				currentOffsetX -= xchange;
			}

			// Force the movie to redraw upon moving:
			myPlayer.redraw(null);
		}
		catch (Exception ee)
		{
			System.out.println(ee);
		}
	}

	void alterSize(int change)									// CHANGE MADE:
	{															// Crunched down to one function.
		try
  		{
			needsRedrawing = true;
            QDRect myRect = myPlayer.getDisplayBounds();
            Rectangle canvasRect = myComponent.getBounds();

            int theWidth = myRect.getWidth();
            int theHeight = myRect.getHeight();
            
            if (change == -1)
            {
	            if (theWidth/2 < canvasRect.width)
	            {
	            	theWidth = canvasRect.width;
	            	theHeight = canvasRect.height;
				}
	            else
	            {
	            	theWidth /= 2;
	            	theHeight /= 2;
				}
			}
			else
			{
				theWidth *= 2;
				theHeight *= 2;
			}
			
			//reset clipping region - shouldn't need this?
			Rectangle myBounds = myComponent.getBounds();
			int clipX = myBounds.x;
			int clipY = myBounds.y;
			int titleHeight = this.getInsets().top;
			int clipWidth = myBounds.width;
			int clipHeight = myBounds.height;
			QDRect clip = new QDRect(clipX,clipY-titleHeight,clipWidth,clipHeight);
			myPlayer.setClip(new Region(clip));
			
			myRect.setWidth(theWidth);
			myRect.setHeight(theHeight);
			if (change < 0) {myRect.setX(myBounds.x); myRect.setY(myBounds.y-titleHeight);currentOffsetX = 0;currentOffsetY = 0;}
            myPlayer.setDisplayBounds(myRect);
            //currentOffsetX = 0;
            //currentOffsetY = 0;
            //myPlayer.setLocation(currentOffsetX,currentOffsetY);
            
            String tempTitle = origTitle;
            this.setTitle(tempTitle + " (" + IJ.d2s(100*myRect.getWidth()/startWidth,1) + "%)");

        	changeZoom();

		}
		catch (Exception ee)
		{
			System.out.println(ee);
		}
	}

	void changeZoom()
	// Reset the movie's zoom by altering its sizing rectangle.
	{
  		try
  		{
			//commented out next 3 lines in favor of using myComponent's dimesions
			//QDRect myRect = myPlayer.getDisplayBounds();
            //int theWidth = myRect.getWidth();
            //int theHeight = myRect.getHeight();
			
			//Rectangle myRect = myComponent.getBounds();
			//double theWidth = myRect.getWidth();
            //double theHeight = myRect.getHeight();
			double theWidth = this.getWidth();
			double theHeight = 0;
			//added 11-13-04
			if (MyPanel.isVisible()){
            theHeight = this.getHeight()-this.getInsets().top-MyPanel.getHeight()-MyScrollBar.getHeight();
			}
			else{
			theHeight = this.getHeight()-this.getInsets().top-MyScrollBar.getHeight()-3;//need slight padding for aesthetics
			}
            //reset aspect ratio here			
			if ((theWidth/startWidth) > (theHeight/startHeight))
			{
			theWidth = startWidth*(theHeight/startHeight);
			}
			else
			{
			theHeight = startHeight*(theWidth/startWidth);
			}
			
			//resize the movie component
			myComponent.setSize((int)theWidth,(int)theHeight);
			//now center on AWT frame
			//code for invivsle control panel added 11-13-04
			if (MyPanel.isVisible()){
			myComponent.setLocation((this.getWidth()-(int)theWidth)/2,(this.getInsets().top+this.getHeight()-MyPanel.getHeight()-MyScrollBar.getHeight()-(int)theHeight)/2);
			}
			else
			{
			myComponent.setLocation((this.getWidth()-(int)theWidth)/2,(this.getInsets().top+this.getHeight()-MyScrollBar.getHeight()-(int)theHeight)/2);			
			}
			//now reposition controls
			MyScrollBar.setSize(myComponent.getWidth(),MyScrollBar.getHeight());
			MyScrollBar.setLocation(myComponent.getLocation().x,myComponent.getLocation().y+myComponent.getHeight()+1);
			int deltaX = 0;
			//center MyPanel under myComponent
			//added 11-13-04
			if (MyPanel.isVisible()) {deltaX = (MyPanel.getWidth() - myComponent.getWidth())/2;}
			//
			if (deltaX > myComponent.getLocation().x) {
			deltaX = myComponent.getLocation().x;
			}
			if (MyPanel.isVisible()) {MyPanel.setLocation(myComponent.getLocation().x-deltaX,MyScrollBar.getLocation().y+MyScrollBar.getHeight()+1);}
            myPlayer.redraw(null);
            String tempTitle = origTitle;
            this.setTitle(tempTitle + " (" + IJ.d2s(100*theWidth/startWidth,1) + "%)");
			
			//Added 1-30-05
			//reset clipping region - shouldn't need this?
			/*
			Rectangle myBounds = myComponent.getBounds();
			int clipX = myBounds.x;
			int clipY = myBounds.y;
			int titleHeight = this.getInsets().top;
			int clipWidth = myBounds.width;
			int clipHeight = myBounds.height;
			QDRect clip = new QDRect(clipX,clipY-titleHeight,clipWidth,clipHeight);
			myPlayer.setClip(new Region(clip));
			*/
			//IJ.log("Component: X=" + myComponent.getLocation().getX() + " Y=" + myComponent.getLocation().getY() + " W=" + myComponent.getWidth() + " H=" + myComponent.getHeight());
			//IJ.log("Player: X=" + myPlayer.getDisplayBounds().getX() + " Y=" + myPlayer.getDisplayBounds().getY() + " W=" + myPlayer.getDisplayBounds().getWidth() + " H=" + myPlayer.getDisplayBounds().getHeight());
			//IJ.log("Clip: X=" + myPlayer.getClip().getBounds().getX() + " Y=" + myPlayer.getClip().getBounds().getY() + " W=" + myPlayer.getClip().getBounds().getWidth() + " H=" + myPlayer.getClip().getBounds().getHeight());
			//IJ.log("Slider: X=" + MyScrollBar.getLocation().x + "Y=" + MyScrollBar.getLocation().y + " W=" + MyScrollBar.getWidth() + " H=" + MyScrollBar.getHeight());
			//IJ.log("***********");
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

	void showMovieInfo()
	// Create a string with movie info and display it in an ImageJ message box.
	{
  		try
  		{
			QDRect myRect = myPlayer.getDisplayBounds();
            int theWidth = myRect.getWidth();
            int theHeight = myRect.getHeight();
            myPlayer.redraw(null);
            IJ.showMessage("Movie info","Movie: "+ movieName + "\n" + "Information: " + infoString + "\n" +"Focal planes: " + IJ.d2s(totalPlanes,0) + "\n" + "Time points: " + IJ.d2s(totalTimePoints/totalPlanes,0) + "\n"+"Spacing (microns): " + spacingString + "\n" + "Time interval (sec): " + timeString + "\n" + "Zoom: " + IJ.d2s(100*theWidth/startWidth,1) + "%");
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
		}
	}

	void StopPlay()
	// Set the movie's rate to zero.
	{
		try
		{
			needsRedrawing = true;
			float myRate = 0;            
            m.setRate(myRate);
                        
			findNewPosition();
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

	void PlayForward()
	// Toggle the movie's rate between one and zero (play and stop).
	{
    	try
    	{
			needsRedrawing = true;
			float myRate = 1;            
		    float myOldRate = m.getRate();
		    
		    myRate = ((myOldRate + 1) % 2);			// CHANGE MADE:
           // myComponent.requestFocus();				// This replaced the need for a complicated logic gate
            if (myOldRate == 1) findNewPosition();	// structure, increasing efficiency slightly with math.
            
            /*
			if (myOldRate == 1)
			{
				myRate = 0;
				myComponent.requestFocus();

				// We need to calculate new position since we've been playing the movie:
				findNewPosition();
			}
			if (myOldRate == 0)
			{
				myRate = 1;
				myComponent.requestFocus();
			}
			*/
            
            m.setRate(myRate);
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

    void changeMag(MouseEvent e)
    // Alter the movie magnification by grabbing mag values from the array.
    {
        try
        {
			Container myContainer = myComponent.getParent();

	    	if (e.isControlDown())
	    	{
	    		int tempWidth = (int) origDim.width;
	    		int tempHeight = (int) origDim.height;
	    	    currentMag = 4;
	      		pack();
	      	}
	      	else
	      	{
	      		currentMag = currentMag + 1;
	      		if (currentMag > 19) currentMag = 19;//9
	      		if (currentMag < 0) currentMag = 0;//4
	    		double theWidth = origDim.width;
	    		double theHeight = origDim.height;

	     		int newWidth = (int) (theWidth * magArray[currentMag]);
	     		int newHeight = (int) (theHeight * magArray[currentMag]);
	     
	    		pack();
	      	}
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

	void changeMagMenu(int MyChoice)
	// Change the magnification menu to show the current mag value.
	{
        try
        {
			Container myContainer = myComponent.getParent();
		    int oldMag = currentMag;
	      	currentMag = MyChoice;
	      	
	      	if (currentMag > 19) currentMag = 19;
	      	if (currentMag < 0) currentMag = 0;
	    	
	    	double theWidth = origDim.width;
	    	double theHeight = origDim.height;
			int newWidth = (int)(theWidth * magArray[currentMag]);
	     	int newHeight = (int)(theHeight * magArray[currentMag]);
	     	Dimension theScreen = Toolkit.getDefaultToolkit().getScreenSize();
	     	double screenWidth = theScreen.width;
	     	double screenHeight = theScreen.height;
         	
         	if ((newWidth > (int)screenWidth) || (newHeight > (int)screenHeight))
         	{
         		IJ.showMessage("Movie dimensions exceed screen dimensions. Make another selection.");
         	}
         	else
         	{
	     		pack();
	    	}
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

//Added code 1-31-05 courtesy of Chris Adamson
	void grabFrame (String myWindowTitle)
	// Get the current frame and store it in the global "img" variable.
	{
		try
		{
			//New code
			//if the movie was playing, stop it!
            if (m.getRate() > 0) {
                m.stop();
            }
			
			// set up graphics importer
            GraphicsImporter gi = new GraphicsImporter (StdQTConstants.kQTFileTypePicture);
            GraphicsImporterDrawer gid = new GraphicsImporterDrawer (gi);
			//IJ.log("Stopped movie.");

//OLD CODE FOLLOWS; KNOWN BUGS WITH APPLE CODE IN QTJ 6.1!!
/*
			needsRedrawing = true;
			int time = myPlayer.getTime();
		    m.setTimeValue(time);
                        
			QTImageProducer qtip2 = new QTImageProducer (myPlayer, new Dimension(width,height));
			img = Toolkit.getDefaultToolkit().createImage(qtip2);
			qtip2.redraw(null);
			qtip2.updateConsumers (null);
			
        	ImageProcessor ip = new ColorProcessor(img);
            if (grayscale)
            {
            	ip = ip.convertToByte(false);
            }
            
        	double movieTime = getTimeSinceStart();
			**/
            // take a pict
            Pict pict = m.getPict (m.getTime());
			//IJ.log("Made PICT.");
            
            // add 512-byte header that pict would have as file
            byte[] newPictBytes =
                new byte [pict.getSize() + 512];
            pict.copyToArray (0,
                              newPictBytes,
                              512,
                              newPictBytes.length - 512);
            pict = new Pict (newPictBytes);
			//IJ.log("Added 512K header.");
            
            // export it
            DataRef ref = new DataRef (pict,
                                      StdQTConstants.kDataRefQTFileTypeTag,
                                      "PICT");
            gi.setDataReference (ref);
            QDRect rect = gi.getSourceRect ();
            Dimension dim = new Dimension (rect.getWidth(),
                                           rect.getHeight());
            QTImageProducer qtip2 = new QTImageProducer (gid, dim);
			//IJ.log("Made QTImageProducer. Dimensions: W=" + rect.getWidth() + " H=" + rect.getHeight());
            
            // convert from MoviePlayer to java.awt.Image
            img = Toolkit.getDefaultToolkit().createImage (qtip2);
			//IJ.log("Made AWT Image.");
			//if (img == null) {IJ.log("Unfortunately, image was null.");}
			
            // make an ImageJ image out it
			ImageProcessor ip = new ColorProcessor(img);
			//IJ.log("Made ImageJ ImageProcessor.");
			//if (ip == null) {IJ.log("Unfortunately, image was null.");}

            if (grayscale)
            {
            	ip = ip.convertToByte(false);
            }

			
        	if (stackNeedsReplacing)
        	{
				stackNeedsReplacing = false;

				// Make a new stack with the proper dimensions:
				stack = new ImageStack(rect.getWidth(), rect.getHeight());
				//IJ.log("Made new ImageStack.");
				
				stack.addSlice(null ,ip);
				new ImagePlus(myWindowTitle, stack).show();
				imp = WindowManager.getCurrentImage();
				//IJ.log("Got corresponding ImagePlus.");
			}
			else
			{
            	stack.addSlice(null ,ip);
				//IJ.log("Added slice.");
            }
				imp.setStack(null, stack);
				imp.setSlice(stack.getSize());
                imp.show();
        }
        catch (Exception ee)
        {
        	System.out.println(ee);
        }
	}
	    
    void cleanUpCopy() throws QTException
    {
    }

//Added 1-31-05; from Chris Adamson, "QuickTime for Java Developers' Toolkit"
    public void grabOneFrame() {
        try {
			//if the movie was playing, stop it!
            if (m.getRate() > 0) {
                m.stop();
            }
			
			// set up graphics importer
            GraphicsImporter gi = new GraphicsImporter (StdQTConstants.kQTFileTypePicture);
            GraphicsImporterDrawer gid = new GraphicsImporterDrawer (gi);

            // take a pict
            Pict pict = m.getPict (m.getTime());
            
            // add 512-byte header that pict would have as file
            byte[] newPictBytes =
                new byte [pict.getSize() + 512];
            pict.copyToArray (0,
                              newPictBytes,
                              512,
                              newPictBytes.length - 512);
            pict = new Pict (newPictBytes);
            
            // export it
            DataRef ref = new DataRef (pict,
                                      StdQTConstants.kDataRefQTFileTypeTag,
                                      "PICT");
            gi.setDataReference (ref);
            QDRect rect = gi.getSourceRect ();
            Dimension dim = new Dimension (rect.getWidth(),
                                           rect.getHeight());
            QTImageProducer qtip = new QTImageProducer (gid, dim);
            
            // convert from MoviePlayer to java.awt.Image
            Image image = Toolkit.getDefaultToolkit().createImage (qtip);
			
            // make an ImageJ image out it
			ImageProcessor ip = new ColorProcessor(image);
        	double movieTime = getTimeSinceStart();
        	String title = new String("Time= "+currentTime + " Plane= " + currentPlane);
			ImagePlus imp = new ImagePlus(title, ip);
			imp.show();

        } catch (QTException qte) {
            System.out.println(qte);
        }
    } 

/*
THIS NO LONGER WORKS RELIABLY; KNOWN BUG IN QTJ 6.1!
	void grabOneFrame()
	// Grab the frame at the current time point and focal plane.
	{
		try
		{
			needsRedrawing = true;
			int time = myPlayer.getTime();            
			m.setTimeValue(time);
			QTImageProducer qtip2 = new QTImageProducer(myPlayer, new Dimension(width,height));
			img = Toolkit.getDefaultToolkit().createImage(qtip2);
			qtip2.redraw(null);
			qtip2.updateConsumers (null);
			ImageProcessor ip = new ColorProcessor(img);
        	double movieTime = getTimeSinceStart();
        	String title = new String("Time= "+currentTime + " Plane= " + currentPlane);

			ImagePlus imp = new ImagePlus(title, ip);
			imp.show();
        }
        catch (Exception ee)
        {
        	System.out.println(ee);
        }
	}
**/

	void copyFocalPlane() throws QTException
	// Copy all of the time points at the current focal plane to an IJ stack.
	{      
    	YesNoCancelDialog ync = new YesNoCancelDialog(IJ.getInstance(),
		"Grayscale?", "Copy as 8-bit grayscale? (Less memory by a factor of 4.)");
		
		if (ync.cancelPressed())
		{
			return;
		}
		grayscale = ync.yesPressed();
                        
        QT4DRunner myRunner = new QT4DRunner("focal plane",this);
        myRunner.start();
	}

 	void copyTimePoint() throws QTException
 	// Cope all of the focal planes at the current time point to an IJ stack.
 	{    
    	YesNoCancelDialog ync = new YesNoCancelDialog(IJ.getInstance(),
		"Grayscale?", "Copy as 8-bit grayscale? (Less memory by a factor of 4.)");
		
		if (ync.cancelPressed())
		{ 
        	return;
        }
		grayscale = ync.yesPressed();

        
        QT4DRunner myRunner = new QT4DRunner("time point",this);
        myRunner.start();
	}

	public void windowClosing(WindowEvent e)
	// If the user tries to close the window, make sure QT gets shut down.
	{
		shutDown();
	}

	public void windowClosed (WindowEvent e)
	{
	}
	
    public void focusGained(FocusEvent e) {
       // IJ.log("Focus gained: " + e);
    }
    
    public void focusLost(FocusEvent e) {
       // IJ.log("Focus lost: " + e);
    }

	public void actionPerformed(ActionEvent e)
	// Detect and handle user interactions with buttons and other window objects.
	{
		String cmd=e.getActionCommand();
        if (cmd !=null) doCommand(cmd);
		if ((e.getSource()==StepFwdButton) && (e.getActionCommand()!= ">>"))
		{
			stepTimePoint(true);
        }

		if (e.getSource()==StepBwdButton)
		{
			if (e.getActionCommand()!= "<<")
			{
				stepTimePoint(false);
			}
		}

		if (e.getSource()==MyDownButton)
		{
			if (e.getActionCommand()!= "v")
			{
		    	stepFocalPlane(false);
			}
		}

		if (e.getSource()==MyUpButton)
		{
			if (e.getActionCommand()!= "^")
			{
				stepFocalPlane(true);
			}
		}

		if (e.getSource()==HomeButton)	GoFirst();
		if (e.getSource()==EndButton)	GoLast();
		if (e.getSource()==PlayButton)	PlayForward();
    	myComponent.requestFocus();
	}
	
	public void mouseWheelMoved(MouseWheelEvent e)
	// Handle mouse scroll wheel movement.
	{
		if ((e.getSource()==myComponent) || (e.getSource()==this))
		{
		if (e.getUnitsToScroll() < 0) //moved the scroll wheel up
		{
		if (e.isAltDown()) {stepTimePoint(true);} else {stepFocalPlane(true);}
		} else //moved it down
		{
		if (e.isAltDown()) {stepTimePoint(false);} else {stepFocalPlane(false);}
		}
		//IJ.showStatus("Scroll direction: " + direction + " Scroll amount: " + e.getUnitsToScroll());
		}
    }


	public void mouseMoved(MouseEvent e)
	// Handle mouse movement and alter the cursor if necessary.
	{
		if (e.getSource()==myComponent)
		{
	    	int myX = e.getX();
            int myY = e.getY();
            int theID = Toolbar.getToolId();
            Dimension mySize = myComponent.getSize();
            int myWidth = mySize.width;
            int myHeight = mySize.height;
            
            if (myY < 40) this.setCursor(UpCursor);
            if (myX < 40) this.setCursor(LeftCursor);
            if (myY > (myHeight - 40)) this.setCursor(DownCursor);
            if (myX > (myWidth - 40)) this.setCursor(RightCursor);
            if ((myX > 40) && (myX < (myWidth -40)) && (myY > 40) && (myY < (myHeight - 40))) this.setCursor(ArrowCursor);
            if ((theID == 12) && (myX > 40) && (myX < (myWidth -40)) && (myY > 40) && (myY < (myHeight - 40))) this.setCursor(MoveCursor);
		}
    }

    public void mouseDragged(MouseEvent e)
    // Handle the mouse being dragged, and see if resizing is needed.
    {
    	try
    	{
	 		if (e.getSource() == myComponent)
	 		{
         		int theID = Toolbar.getToolId();
      	 		if (theID == 12)
      	 		{
      				int x = e.getX();
      				int y = e.getY();
            		if (!startedDragging)
            		{
            			startedDragging = (!startedDragging);
            			dragOffsetX = x - currentOffsetX ; dragOffsetY = y - currentOffsetY;
            		}
            		
           			oldOffsetX = currentOffsetX;
           			oldOffsetY = currentOffsetY;
            		currentOffsetX = x - dragOffsetX;
            		currentOffsetY = y - dragOffsetY;
           			
           			Rectangle canvasRect = myComponent.getBounds();
           			QDRect myRect = myPlayer.getDisplayBounds();

            		if (!(currentOffsetX <= 0))
              		{
              			currentOffsetX = oldOffsetX;
              		}

            		if (!(currentOffsetY <= 0))
               		{
               			currentOffsetY = oldOffsetY;
               		}

            		if (!((myRect.getWidth() + currentOffsetX) >= canvasRect.width))
                	{
                		currentOffsetX = oldOffsetX;
                	}

            		if (!(currentOffsetY <= 0))
              		{
              			currentOffsetY = oldOffsetY;
              		}
              		
            		if (!((myRect.getHeight() + currentOffsetY) >= canvasRect.height))
            		{
            			currentOffsetY = oldOffsetY;
            		}

					// If the movie was dragged, change its offset within the window:
					//Had to make changes due to relative coordinate systems in following
            		myPlayer.setLocation(currentOffsetX + canvasRect.x, currentOffsetY + canvasRect.y - this.getInsets().top);
            		needsRedrawing = true;
            		myPlayer.redraw(null);
				}
        	}
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
		}
	}

    public void mousePressed(MouseEvent e)
    // If the mouse is pressed, see if the user clicked a button or the movie.
    // If the user clicked a button, trigger its event handler.
    // If the user clicked the movie, change the current focal plane or time point.
    {
    	int x=e.getX(), y=e.getY();
        Dimension mySize = myComponent.getSize();
        int myWidth = mySize.width;
        int myHeight = mySize.height;
        mousePressObject = e.getSource();
       	
       	if ((e.getSource()==StepBwdButton) || (e.getSource()==StepFwdButton) || (e.getSource()==MyUpButton) || (e.getSource()==MyDownButton) || (e.getSource()==myComponent))
       	{
       		stopPlaying();
         	mousePressX = x;
         	mousePressY = y;
         	
     		if ((e.isPopupTrigger() || e.isMetaDown()) && (e.getSource()==myComponent) && (Toolbar.getToolId() != 11))
     		{
        		pm.show(e.getComponent(),x,y);
         	}
         	else
         	{
    			// We need to figure out how to detect if the mouse is still down!
      			mouseIsDown = true;
      			
      			if ((e.getModifiers() & e.SHIFT_MASK) != 0)
      			{
	                if (mousePressObject == StepFwdButton) 		{ GoLast(); }
	                else if (mousePressObject == StepBwdButton) { GoFirst(); }
	                else if (mousePressObject == MyUpButton) 	{ extremeFocalPlane(true); }
	                else if (mousePressObject == MyDownButton) 	{ extremeFocalPlane(false); }
	                if (myTimer != null)
	                {
	                	myTimer.stop();						            
						// Make sure that focus goes back to the canvas:
						myComponent.requestFocus();
	                }
                }
       			else if (myTimer==null)
       			{
              		// Create a timer if one doesn't exist. First do one click's worth!
              		if  (mousePressObject == myComponent)
       				{
                		if (mousePressY < 40) stepFocalPlane(true);
                        if (mousePressX < 40) stepTimePoint(false);
                        if (mousePressY > (myHeight - 40)) stepFocalPlane(false);
                        if (mousePressX > (myWidth - 40)) stepTimePoint(true);
                    }
                    else if (mousePressObject==StepBwdButton)	{ stepTimePoint(false); }
                    else if (mousePressObject==StepFwdButton)	{ stepTimePoint(true); }
                    else if (mousePressObject==MyUpButton)		{ stepFocalPlane(true); }
                    else if (mousePressObject==MyDownButton)	{ stepFocalPlane(false); }
                    
       				int ONE_TENTH = 100;
        			myTimer = new javax.swing.Timer(ONE_TENTH, new ActionListener()
        			{
            			public void actionPerformed(ActionEvent evt)
            			{
                			if ((myTimer.isRunning()) && (mouseIsDown))
                			{
                				if  (mousePressObject == myComponent)
                				{
                					Dimension theSize = myComponent.getSize();
                            		int theWidth = theSize.width;
                            		int theHeight = theSize.height;
                            		
                            		if (mousePressY < 40)				stepFocalPlane(true);
                            		if (mousePressX < 40)				stepTimePoint(false);
                            		if (mousePressY > (theHeight - 40))	stepFocalPlane(false);
                            		if (mousePressX > (theWidth - 40))	stepTimePoint(true);
                            	}
                            	else if  ((mousePressObject==StepBwdButton) || (mousePressObject==StepFwdButton) || (mousePressObject==MyUpButton) || (mousePressObject==MyDownButton))
                            	{
                            		if (mousePressObject==StepBwdButton)		{ stepTimePoint(false); }
                               		else if (mousePressObject==StepFwdButton)	{ stepTimePoint(true); }
                               		else if (mousePressObject==MyUpButton)		{ stepFocalPlane(true); }
                               		else if (mousePressObject==MyDownButton)	{ stepFocalPlane(false); }
                            	}
                			}
                		}
              		});
              		
              		myTimer.start();
            	}
            	else
            	{
            		// It was just a button click, so pass the event!
            		if  (mousePressObject == myComponent)
            		{
                    	if (mousePressY < 40)				stepFocalPlane(true);
                        if (mousePressX < 40)				stepTimePoint(false);
                        if (mousePressY > (myHeight - 40))	stepFocalPlane(false);
                        if (mousePressX > (myWidth - 40))	stepTimePoint(true);						            
						// Make sure that focus goes back to the canvas:
						myComponent.requestFocus();
                    }
                    else if (mousePressObject==StepBwdButton)	{ stepTimePoint(false); }
                    else if (mousePressObject==StepFwdButton)	{ stepTimePoint(true); }
                    else if (mousePressObject==MyUpButton)		{ stepFocalPlane(true); }
                    else if (mousePressObject==MyDownButton)	{ stepFocalPlane(false); }					            
					// Make sure that focus goes back to the canvas:
					myComponent.requestFocus();
                	myTimer.restart();
                }
     		}
		}
  	}
  	
    public void mouseReleased(MouseEvent e)
    // The mouse was let up, so make sure the button timer knows that no buttons are currently being held down.
    {
       	mouseIsDown = false;

       	if (myTimer!=null)
       	{
       		myTimer.stop();
		}

       	startedDragging = false; mousePress = false;		            
		// Make sure that focus goes back to the canvas:
		myComponent.requestFocus();
    }

    public void mouseEntered(MouseEvent e)
    {
    }

    public void mouseExited(MouseEvent e)
    // Change the cursor to the system default when the user leaves the window.
    {
    	if (e.getSource() == myComponent)
    	{
    		this.setCursor(ArrowCursor);
    	}
    }

    public void mouseClicked(MouseEvent e)
    // Handle mouse clicks within the main user interface.
    {
    	try
    	{
        	if (e.getSource()==myComponent)
        	{
            	int theID = Toolbar.getToolId();
                Dimension mySize = myComponent.getSize();
                int myWidth = mySize.width;
                int myHeight = mySize.height;
                int myX = e.getX();
                int myY = e.getY();
              
				// If the magnifying glass tool is selected in the toolbar, then zoom in:
                if (theID == 11)
                {
				/*
	        		if (!((myY < 40) || (myX < 40) || (myY > (myHeight - 40)) || (myX > (myWidth - 40))))
                    {
                         if ((e.isMetaDown()) || (e.isAltDown()) || (e.isControlDown()))
                         {
                         	alterSize(-1);
                         }
                         else
                         {
                         	alterSize(1);
                         }
					}
				*/
				}
                else if (e.isShiftDown() & (theID != 11))
                {
				/*
                	if ((myX > 40) && (myX < (myWidth -40)) && (myY > 40) && (myY < (myHeight -40)))
                	{
                    	int tempWidth = (int) origDim.width;
	                    int tempHeight = (int) origDim.height;
	    	            currentMag = origMag;
	                    changeMagMenu(currentMag);
	                    pack();
					}
				*/
				}
			}
		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
	}

 	public void keyPressed(KeyEvent e)
 	// A key was pressed! Check if it was a hot key, and if the shift key is being pressed.
 	{
 		int flags = e.getModifiers();
		
		switch (e.getKeyCode())
		{
            case 65:
                alterPosition(-10,0);
                break;
                
            case 83:
                alterPosition(10,0);
                break;
                
            case 88:
                alterPosition(0,10);
                break;
                
            case 87:
                alterPosition(0,-10);
                break;
                
			case KeyEvent.VK_UP:
				if (flags == 1)
				{
					extremeFocalPlane(true);
				}
				else
				{
					stepFocalPlane(true);
				}
				break;
				
			case KeyEvent.VK_DOWN:
				if (flags == 1)
				{
					extremeFocalPlane(false);
				}
				else
				{
					stepFocalPlane(false);
				}
				break;
				
			case KeyEvent.VK_LEFT:
				if (flags == 1)
				{
					GoFirst();
				}
				else
				{
					stepTimePoint(false);
				}
				break;
				
			case KeyEvent.VK_RIGHT:
				if (flags == 1)
				{
					GoLast();
				}
				else
				{
					stepTimePoint(true);
				}
				break;
				
			case KeyEvent.VK_SPACE:
				//if (!spaceBarPressed)
	           // {
	            	//spaceBarPressed = true;
					PlayForward();
					/*
	            	tempID = Toolbar.getToolId();
	            	int myID = Toolbar.HAND;
	            	if (tempID != Toolbar.HAND)
	            	{
	            		IJ.setTool(myID);
	            	}
					*/
	           // }
				break;
				
			case KeyEvent.VK_ENTER:
				//PlayForward();
				break;
		}
	}

	public void keyReleased(KeyEvent e)
	{
     	int flags = e.getModifiers();
     	
     	if (e.getKeyCode() == 32)
     	{
			// The space bar was pressed, so switch tools in the ImageJ menu!
			//spaceBarPressed = false;
            //IJ.setTool(tempID);
		}
	}
	
	public void keyTyped(KeyEvent e)
	{
	}

	public void componentResized(ComponentEvent e)
	// Redraw the window if it is resized by the user.
	{
		int currentWidth = getWidth();
        int currentHeight = getHeight();
        
       // try
       // {		
			changeZoom();
			/*
			Rectangle myBounds = myComponent.getBounds();
            int clipX = myBounds.x; int clipY = myBounds.y; int clipWidth = myBounds.width; int clipHeight = myBounds.height;
            QDRect clip = new QDRect(clipX,clipY,clipWidth,clipHeight);
            myPlayer.setClip(new Region(clip));
			**/
       // }
       // catch (QTException ee)
       // {
		//	ee.printStackTrace();
		//}
		
        
		currentOffsetX = 0; currentOffsetY = 0;
		/*
        // Try spawning a timer that checks for resizing of the qtcanvas:
   		Date myInitialDate = new Date();
   		int ONE_HALF = 500;
   		
   		if (resizeTimer == null)
   		{
    		resizeTimer = new javax.swing.Timer(ONE_HALF, new ActionListener()
    		{
        		public void actionPerformed(ActionEvent evt)
        		{
        			Date myCurrentDate = new Date();
					//next line added for testing only!
					changeZoom();
					//
        			resizeTimer.stop();
        		}
        	});
        	
          	resizeTimer.start();
        }
        else
        {
        	resizeTimer.restart();
        }
		**/
    }

	public void componentMoved(ComponentEvent e)
	{
    }

	public void componentShown(ComponentEvent e)
	{
	}

	public void componentHidden(ComponentEvent e)
	{
	}

    void doCommand(String cmd)
    // Handle all of the commands generated by both the main and the pop-up menus.
    {
    	try
    	{
        	if (cmd==null)
            return;
            
        	if (cmd.equals("About This Plugin..."))
        	{
        		IJ.showMessage("About QT4D Player...","Version 1.0 beta. A plugin for viewing 4d QuickTime movies within ImageJ." + "\n"+ "Contact the author at jdhardin@facstaff.wisc.edu for more information.");
        	}
        	else if (cmd.equals("Close"))
        	{
        		dispose();
        	}
        	else if (cmd.equals("Copy Focal Plane to Stack"))
        	{
        		copyFocalPlane();
        	}
            else if (cmd.equals("Copy Time Point to Stack"))
            {
        		copyTimePoint();
        	}
        	else if (cmd.equals("Copy Frame to Image Window"))
        	{
        		grabOneFrame();
        	}
        	else if (cmd.equals("Reset Window to Original Size"))
        	{
			
			//Ah, for the good old days when Java was 1.3x and QTJ was 6.0; everything worked fine then!
            	//old code follows
				/*
				int tempWidth = (int) origDim.width;
	            int tempHeight = (int) origDim.height;
	            QDRect myRect = myPlayer.getDisplayBounds();
                myRect.setWidth(tempWidth);
                myRect.setHeight(tempHeight);
                myPlayer.setDisplayBounds(myRect);
                currentMag = origMag;
                changeMagMenu(currentMag);
                String tempTitle = origTitle;
                this.setTitle(tempTitle + " (" + IJ.d2s(100*myRect.getWidth()/startWidth,1) + "%)");
				**/
				
			//The following should work, but of course it broke under JVM 1.4.2 and QTJ 6.1 under OS X.
			//Everything resized but the AWT frame.
			//Under JVM 1.5 it seems to work again!
				
				//reset locations and sizes of movie container, slider, buttons, and frame to original sizes
				myComponent.setLocation(origMovieLocation);
				MyScrollBar.setLocation(origScrollLocation);
				MyPanel.setLocation(origPanelLocation);
				myComponent.setSize(origComponentSize);
				MyScrollBar.setSize(origScrollSize);
				//IJ.log("Current size: " + this.getSize().width + "," + this.getSize().height);
				//IJ.log("Original size: " + origFrameSize.width + "," + origFrameSize.height);
				this.setSize(origFrameSize);this.setSize(origFrameSize);
				
				
			//The following is inelegant, but works sometimes for me with JVM 1.4.2.05 and QTJ 6.1 under OS X.
			//The hiding and packing seems to do the trick. Why? I have no earthly idea...	
				//this.repaint();
				/*
				myComponent.setVisible(false);
				myComponent.setSize(origComponentSize);
				MyScrollBar.setVisible(false);
				MyPanel.setVisible(false);
				pack();
				this.setSize(origFrameSize);
				myComponent.setVisible(true);
				MyScrollBar.setVisible(true);
				MyPanel.setVisible(true);
				//pack();
				changeZoom();
				MyPanel.repaint();
				MyScrollBar.repaint();
				myComponent.repaint();
				this.repaint();
				*/
        	}
			//Added 11-13-04
			else if (cmd.equals("Show/Hide Control Panel"))
        	{
			if (MyPanel.isVisible()) 
			{
			myComponent.setLocation(0,0);
			MyScrollBar.setLocation(myComponent.getLocation().x,myComponent.getLocation().y+myComponent.getHeight()+1);
			setSize(myComponent.getSize().width,this.getSize().height-MyPanel.getSize().height);
			
			}
			else 
			{
			if (this.getSize().width < MyPanel.getSize().width)
			{
			setSize(MyPanel.getSize().width,this.getSize().height+MyPanel.getSize().height+1);
			}
			else
			{
			setSize(this.getSize().width,this.getSize().height+MyPanel.getSize().height+1);
			}
			myComponent.setLocation((int)(this.getWidth())/2,0);
			MyScrollBar.setLocation(myComponent.getLocation().x,myComponent.getLocation().y+myComponent.getHeight()+1);						
			}

			MyPanel.setVisible(!MyPanel.isVisible());
			//pack();	
			}
			//
        	else if (cmd.equals("Get Info..."))
        	{
        		showMovieInfo();
        	}
			else if (cmd.equals("Apply Filter..."))
        	{
        		FilterBuilder();
        	}
        	else if (cmd.equals("Revert to Original"))
        	{
        		RevertFilters();
        	}

		}
		catch (Exception ee)
		{
        	System.out.println(ee);
        }
    }

	public void itemStateChanged(ItemEvent ie)
	// Handle any changes in user menu choices.
	{
		Choice c = (Choice) ie.getItemSelectable();
		changeMagMenu(c.getSelectedIndex());
	}

	public void windowIconified(WindowEvent e)
	{
	}
	
	public void windowDeiconified(WindowEvent e)
	{
	}
	
	public void windowActivated(WindowEvent e)
	{
	}
	
	public void windowDeactivated(WindowEvent e)
	{
	}

    public void adjustmentValueChanged(AdjustmentEvent ae)
    {
    	Scrollbar MyScrollBar = (Scrollbar)ae.getAdjustable();
    	SlideMovie (MyScrollBar.getValue());
		//IJ.log("Valued changed: " + MyScrollBar.getValue());
    	myComponent.requestFocus();
    }

	public int execute (Movie m)
	{
		try
		{
			qtip.updateConsumers (null);
		}
		catch (QTException e)
		{
			return e.errorCode();
		}
		return 0;
	}
}

// ************************************************************************
// The following is courtesy of Apple QTJava demo code:

	/**
	 * This class implements a method that is called when the movie stops
	 */
	 
	class QT4DMonitor extends quicktime.std.clocks.ExtremesCallBack
	{
		QT4DMovie myQT4DMovie;

		public QT4DMonitor(TimeBase tb, int flag, QT4DMovie theQT4DMovie) throws QTException
		{
			super(tb, flag);
			myQT4DMovie = theQT4DMovie;
		}

   		public void execute()
   		{
			try
			{
				// We need to calculate position since we've been playing the movie:
	            float myRate = 0;
	            myQT4DMovie.m.setRate(myRate);
	            myQT4DMovie.findNewPosition();
		  	    cancel();
				callMeWhen ();
			}
			catch (QTException e)
			{
			}
		}
	}

	class QT4DTickler implements Ticklish
	{

		public QT4DMovie myQT4DMovie;
        public int myTime;

		public QT4DTickler(QT4DMovie theQT4DMovie) throws QTException
		{
			myQT4DMovie = theQT4DMovie;
		}

		public void timeChanged(int newTime) throws QTException
		{
		}
		
		public boolean tickle(float er, int time) throws QTException
		{
			myTime = time;
            myQT4DMovie.findRunningPosition();

			return true;
		}
	}
              

 	class QT4DRunner extends Thread
 	{
    	public QT4DMovie myQT4DMovie;
    	public String myString;

    	public QT4DRunner(String str, QT4DMovie theQT4DMovie) throws QTException
    	{
        	super(str);myQT4DMovie = theQT4DMovie;myString = str;
    	}
    	
    	public void run()
    	{
    		if (myString == "time point")
    		{
    			copyTimePoint();
    		}
    		else if (myString == "focal plane")
    		{
    			copyFocalPlane();
    		}
    	}

    	void copyFocalPlane()
    	{
      		try
      		{
			/*
      			if (myQT4DMovie.img == null)
      			{
              		System.out.println("On entry to grabframe, img is null!");
      			}
      		**/	
				int i = 0;
        		IJ.showStatus("Copying frame " + (i+1) + " of " + myQT4DMovie.framesPerFocalPlane);

        		String tempstr = Integer.toString(myQT4DMovie.currentPlane);
                myQT4DMovie.stackNeedsReplacing = true;

				for (i = 0; i < myQT4DMovie.framesPerFocalPlane; i++)
				{
                	int frameNumber = (myQT4DMovie.currentPlane - 1) * myQT4DMovie.framesPerFocalPlane + i + 1;
					myQT4DMovie.m.setTimeValue(myQT4DMovie.frameTime[frameNumber-1]);
	        		myQT4DMovie.grabFrame("Focal plane " + tempstr);
					//Next line added to take care of crashing (Sync problems with memory manager? QTJ? I don't know...)
					IJ.wait(500);
                
                	IJ.showStatus("Copying frame " + (i+1) + " of " + myQT4DMovie.framesPerFocalPlane);
	       		}
	       		
                myQT4DMovie.cleanUpCopy();
	  			myQT4DMovie.stackNeedsReplacing = true;
	  			
	   	  		// We need to return to original frame before copy procedure:
	   	   		int frameNumber1 = (myQT4DMovie.currentPlane - 1) * myQT4DMovie.framesPerFocalPlane + myQT4DMovie.currentTime;
           		myQT4DMovie.m.setTimeValue(myQT4DMovie.frameTime[frameNumber1-1]);
                IJ.showStatus("");
			}
			catch (Exception ee)
			{
            	System.out.println(ee);
            }
		}

     	void copyTimePoint()
     	{
        	try
        	{
			/*
				if(myQT4DMovie.img == null)
				{
					System.out.println("On entry to grabframe, img is null!");
				}
			**/				
				int i = 0;
        		IJ.showStatus("Copying frame " + (i+1) + " of " + myQT4DMovie.totalPlanes);
        		
        		// We have to reset active segment so that we can go to multiple focal planes:
       			myQT4DMovie.stackNeedsReplacing = true;

	    		int frameNumber2 = (myQT4DMovie.totalPlanes - 1) * myQT4DMovie.framesPerFocalPlane + myQT4DMovie.framesPerFocalPlane;
				int myduration = myQT4DMovie.frameTime[frameNumber2 - 1] - myQT4DMovie.frameTime[0] + 1;
				int mystart = myQT4DMovie.frameTime[0];
				TimeInfo mytimeinfo = new TimeInfo(mystart, myduration);
				myQT4DMovie.m.setActiveSegment(mytimeinfo);
        		String tempstr = Integer.toString(myQT4DMovie.currentTime);

				for (i = 0; i < myQT4DMovie.totalPlanes; i++)
				{
					int frameNumber = (i) * myQT4DMovie.framesPerFocalPlane + myQT4DMovie.currentTime;
			        myQT4DMovie.m.setTimeValue(myQT4DMovie.frameTime[frameNumber-1]);
	                myQT4DMovie.grabFrame("Time point " + tempstr);
					//Next line added to take care of crashing (Sync problems with memory manager? QTJ? I don't know...)
					IJ.wait(500);

	        		IJ.showStatus("Copying frame " + (i+1) + " of " + myQT4DMovie.totalPlanes);
	        	}
                myQT4DMovie.cleanUpCopy();
	   	  		myQT4DMovie.stackNeedsReplacing = true;
	   	  
	   	  		// Reset the active segment:
	   	  		myQT4DMovie.setMyMovieLimits();
	   	  
	   	  		//need to return to original frame before copy procedure
	   	   		int frameNumber1 = (myQT4DMovie.currentPlane - 1) * myQT4DMovie.framesPerFocalPlane + myQT4DMovie.currentTime;
           		myQT4DMovie.m.setTimeValue(myQT4DMovie.frameTime[frameNumber1-1]);
           		IJ.showStatus("");
         	}
         	catch (Exception ee)
         	{
                 System.out.println(ee);
         	}
		}
	}


